----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:10:29 04/15/2010 
-- Design Name: 
-- Module Name:    ControlRegister - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ControlRegister is
    Port ( 
			  ctrldata : in  STD_LOGIC_VECTOR (7 downto 0);
           ctrlclk : in  STD_LOGIC;
			  ctrlsel : in STD_LOGIC;
			  ctrladdr : in STD_LOGIC;
			  ctrlrnw : in STD_LOGIC;
           reset : in  STD_LOGIC;
			  
			  control_reg_init : in std_logic_vector (7 downto 0);
			  clock_reg_init : in std_logic_vector (7 downto 0);
			  
			  -- control register
           wprotect : out  STD_LOGIC_VECTOR (1 downto 0);
           hidebogus : out  STD_LOGIC;
           bootrom : out  STD_LOGIC;
           prgrom : out  STD_LOGIC;
			  slow64k : out STD_LOGIC;
			  slowclk : out STD_LOGIC;
			  fastvread : out STD_LOGIC;
			  
			  -- clock control
           clklatch : out  STD_LOGIC_VECTOR (4 downto 0);
			  fastmode : out std_logic;
			  
			  accesstoggle : out std_logic
	);

end ControlRegister;

architecture Behavioral of ControlRegister is
	
		signal accesstoggleint : std_logic;
		
begin
		
		DoWriteAccess: process(ctrlclk, ctrlrnw, reset, ctrlsel, ctrladdr, ctrldata, control_reg_init, clock_reg_init)
		begin
			if (reset = '0') then
			   accesstoggleint <= '0';
				
				-- control register
				wprotect <= control_reg_init(1 downto 0);
				fastvread <= control_reg_init(2);
				hidebogus <= control_reg_init(3);
				slowclk <= control_reg_init(4);
				slow64k <= control_reg_init(5);
				bootrom <= control_reg_init(6);
				prgrom <= control_reg_init(7);
				-- clock control
				clklatch <= clock_reg_init (4 downto 0);
				fastmode <= clock_reg_init (7);
				
			elsif (falling_edge(ctrlclk)) then

				if (ctrlsel = '1') then 
					accesstoggleint <= ctrlrnw; -- not(accesstoggleint);
			 
					if (ctrlrnw = '0') then				
					
					  if (ctrladdr = '1') then
							-- write $00exxf
							wprotect <= ctrldata(1 downto 0);
							fastvread <= ctrldata(2);
							hidebogus <= ctrldata(3);
							slowclk <= ctrldata(4);
							slow64k <= ctrldata(5);
							bootrom <= ctrldata(6);
							prgrom <= ctrldata(7);
					  else
							-- write $00exxe
							if (ctrldata(3 downto 0) <= "00011") then
								clklatch <= "00011";
							else
								clklatch <= ctrldata(4 downto 0);
							end if;
							fastmode <= ctrldata(7);
					  end if;
					end if;
				end if;
			end if;
		end process DoWriteAccess;

		accesstoggle <= accesstoggleint;
		
--		DoReadAccess: process(cpuclkint, phi0D2, reset)
--		begin
--			if (cpuclkint = '0') then 
--				cpudata <= "ZZZZZZZZ";
--				myregister <= '0';
--			elsif (rising_edge(phi0D2) and cpurnw = '1') then
--				if (islow64k = '1'
--						and isio = '1'
--						and cpuaddr_ismyio = '0') then
--					 case cpuaddrin_16 is
--						when "1111" =>
--							-- read $00efff
--							myregister <= '1';
--							cpudata <= (prgrom, bootrom, slow64k, slowonly, wprotect(1), wprotect(0), clkdiv(1), clkdiv(0));
--						when others =>
--							cpudata <= "ZZZZZZZZ";
--							myregister <= '0';
--					end case;
--				end if;
--			end if;
--		end process DoReadAccess;


end Behavioral;

