----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:05:23 08/10/2010 
-- Design Name: 
-- Module Name:    SystemMap - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SystemMap is
    Port ( 
			  cpuaddrin_64k : in  STD_LOGIC_VECTOR (23 downto 16);
           cpuaddrin_2k : in  STD_LOGIC_VECTOR (15 downto 11);
			  cpuaddrin_ismyio : in STD_LOGIC;
			  cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
			  islow64k : in STD_LOGIC;
			  
           isio : out  STD_LOGIC;
           isvideo : out  STD_LOGIC;
           busaccess : out  STD_LOGIC;
           romaccess : out  STD_LOGIC;
			  rammirror : out STD_LOGIC;
			  ctrlsel : out STD_LOGIC
			  );
end SystemMap;

architecture Behavioral of SystemMap is

			signal isio_int : std_logic;
begin
	

		DoMap: process(cpuaddrin_2k, cpuaddrin_64k, islow64k)
		begin
		
			-- second uppermost 1MByte is place for mapped original bus address space (here 16 times 64k)
			-- is set to 1MByte to be compatible with my CS/A65 65816 card, which has 1MByte bus address space
			if (cpuaddrin_64k(23 downto 20) = "1110") then
				busaccess <= '1';
			else
				busaccess <= '0';
			end if;

			-- uppermost 512kByte is system ROM area
			if (cpuaddrin_64k(23 downto 19) = "11111") then
				romaccess <= '1';
			else
				romaccess <= '0';
			end if;

			-- second uppermost 512kByte is mirrored RAM (lowest 512k)
			-- actually only 64k needed, to copy over slow RAM, could be made smaller
			-- i.e. only 64k guaranteed
			if (cpuaddrin_64k(23 downto 19) = "11110") then
				rammirror <= '1';
			else
				rammirror <= '0';
			end if;

			-- PET I/O area (2k, notwithstanding that some PET have 256 byte I/O only)
			if (islow64k = '1' and cpuaddrin_2k = "11101") then 	-- $XXe800-$XXefff
				isio_int <= '1';
			else
				isio_int <= '0';
			end if;

			-- PET video area (4k)
			if (islow64k = '1' and cpuaddrin_2k(15 downto 12) = "1000") then 	-- $XX8xxx
				isvideo <= '1';
			else
				isvideo <= '0';
			end if;

		end process;

		isio <= isio_int;
		
		-- selects two addresses for clock and control registers
		ctrlsel <= isio_int and not(cpuaddrin_ismyio) and 
						cpuaddrin_16(3) and cpuaddrin_16(2) and cpuaddrin_16(1);

end Behavioral;

