----------------------------------------------------------------------------
--
-- 2010-apr-16 A.Fachat
-- 	added VDA/VPA outputs (currently both driven the same way)
--
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;

package fake_6502b is
  component fake_core_6502b
    port (clk           :in std_logic;
          reset         :in std_logic;
          irq_in        :in std_logic;
          nmi_in        :in std_logic;
          addr_pin      :out std_logic_vector (23 downto 0);
          din           :in  std_logic_vector (7 downto 0);
          dout          :out std_logic_vector (7 downto 0);
          dout_oe       :out std_logic;
          we_pin        :out std_logic;
          rd_pin        :out std_logic;
	  vda_out	:out std_logic;
	  vpa_out	:out std_logic
         );
  end component;

end fake_6502b;

----------------------------------------------------------------------------
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.fake_6502b.all;


entity fake_core_6502b is
  port (clk      :in std_logic;
        reset    :in std_logic;
        irq_in   :in std_logic;
        nmi_in   :in std_logic;
        addr_pin :out std_logic_vector (23 downto 0);
        din      :in  std_logic_vector (7 downto 0);
        dout     :out std_logic_vector (7 downto 0);
        dout_oe  :out std_logic;
        we_pin   :out std_logic;
        rd_pin   :out std_logic;
  	vda_out	 :out std_logic;
  	vpa_out	 :out std_logic
       );
end fake_core_6502b;


architecture fake_6502_arch of fake_core_6502b is

  signal counter       	:std_logic_vector (7 downto 0);

  signal rnw		:std_logic;

begin

	Run: process(reset, clk)
	begin
		if (reset='1') then
			addr_pin <= (others=>'1');
			dout	<= (others=>'Z');
			rnw 	<= '1';
			vda_out	<= '0';
			vpa_out	<= '0';
			counter <= "00000000";

		elsif(rising_edge(clk)) then
			case counter is 

			-- two bogus read cycles for init

			when "00000000" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00000001" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';

			-- setup 10MHz

			when "00000010" =>
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000100";
				rnw 	<= '0';
				vda_out	<= '0';
				vpa_out	<= '1';
	
			-- read reset vector

			when "00000011" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';
			when "00000100" =>
				addr_pin <= ("000000001111111111111101");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';

			-- sample opcode

				-- opcode
			when "00000101" =>
				addr_pin <= ("000000001111110100010110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '1';
				-- parameter
			when "00000110" =>
				addr_pin <= ("000000001111110100010111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';
				-- bogus
			when "00000111" =>
				addr_pin <= ("000000001111110100011000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- read param from ROM
			when "00001000" =>
				addr_pin <= ("000000000000000000000000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- write stack (twice), read stack (twice)

			when "00001001" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001010" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= "01010101";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001011" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001100" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- write stack (twice), read stack (twice)

			when "00001101" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001110" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= "01010101";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001111" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010000" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- write stack (twice), read stack (twice)

			when "00010001" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010010" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= "01010101";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010011" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010100" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- write stack (twice), read stack (twice)

			when "00010101" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010110" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= "01010101";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00010111" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00011000" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';


			-- others

			when others =>
				addr_pin <= ("000000001111111111111101");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			end case;
			counter <= counter + 1;
		end if;
	end process;

	we_pin <= not(rnw);
	rd_pin <= rnw;
	dout_oe <= not(rnw);

end fake_6502_arch;

----------------------------------------------------------------------------
----------------------------------------------------------------------------

